@@ -1,11 +1,14 @@
 """Sphinx extension for linking code examples to reference documentation."""
 
+from pathlib import Path
+
 from sphinx.application import Sphinx
 
 from .extension import SphinxCodeAutoLink, backref, directive
 from .extension.block import clean_ipython, clean_pycon
+from .extension.translation import MESSAGE_CATALOG_NAME
 
-__version__ = "0.16.2"
+__version__ = "0.17.4"
 
 
 def setup(app: Sphinx):
@@ -43,6 +46,18 @@ def setup(app: Sphinx):
         rebuild="html",
         types=[bool],
     )
+    app.add_config_value(
+        "codeautolink_warn_on_no_backreference",
+        default=False,
+        rebuild="html",
+        types=[bool],
+    )
+    app.add_config_value(
+        "codeautolink_warn_on_default_parse_fail",
+        default=False,
+        rebuild="html",
+        types=[bool],
+    )
 
     app.add_directive("autolink-concat", directive.Concat)
     app.add_directive("autolink-examples", directive.Examples)
@@ -51,7 +66,7 @@ def setup(app: Sphinx):
 
     app.connect("builder-inited", state.build_inited)
     app.connect("autodoc-process-docstring", state.autodoc_process_docstring)
-    app.connect("doctree-read", state.parse_blocks)
+    app.connect("doctree-read", state.parse_blocks, priority=490)
     app.connect("env-merge-info", state.merge_environments)
     app.connect("env-purge-doc", state.purge_doc_from_environment)
     app.connect("env-updated", state.create_references)
@@ -64,4 +79,8 @@ def setup(app: Sphinx):
     app.add_node(
         backref.SummaryNode, html=(backref.visit_summary, backref.depart_summary)
     )
+
+    locale_dir = Path(__file__).resolve().parent / "locale"
+    app.add_message_catalog(MESSAGE_CATALOG_NAME, locale_dir)
+
     return {"version": __version__, "env_version": 1, "parallel_read_safe": True}
