@@ -7,6 +7,7 @@ from functools import wraps
 from pathlib import Path
 from traceback import print_exc
 
+from sphinx import version_info as sphinx_version
 from sphinx.ext.intersphinx import InventoryAdapter
 from sphinx.util import import_object
 
@@ -74,6 +75,9 @@ class SphinxCodeAutoLink:
         self.inventory_map: dict[str, str] = {}
         self.warn_missing_inventory = None
         self.warn_failed_resolve = None
+        self.warn_no_backreference = None
+        self.warn_default_parse_fail = None
+        self.highlight_lang = None
 
         # Populated once
         self.outdated_docs: set[str] = set()
@@ -86,7 +90,7 @@ class SphinxCodeAutoLink:
     @print_exceptions()
     def build_inited(self, app) -> None:
         """Handle initial setup."""
-        if app.builder.name != "html":
+        if app.builder.name not in ("html", "dirhtml"):
             self.do_nothing = True
             return
 
@@ -103,6 +107,11 @@ class SphinxCodeAutoLink:
         self.inventory_map = app.config.codeautolink_inventory_map
         self.warn_missing_inventory = app.config.codeautolink_warn_on_missing_inventory
         self.warn_failed_resolve = app.config.codeautolink_warn_on_failed_resolve
+        self.warn_no_backreference = app.config.codeautolink_warn_on_no_backreference
+        self.warn_default_parse_fail = (
+            app.config.codeautolink_warn_on_default_parse_fail
+        )
+        self.highlight_lang = app.config.highlight_language
 
         # Append static resources path so references in setup() are valid
         app.config.html_static_path.append(
@@ -136,6 +145,8 @@ class SphinxCodeAutoLink:
             global_preface=self.global_preface,
             custom_blocks=self.custom_blocks,
             concat_default=self.concat_default,
+            default_highlight_lang=self.highlight_lang,
+            warn_default_parse_fail=self.warn_default_parse_fail,
         )
         doctree.walkabout(visitor)
         self.cache.transforms[visitor.current_document] = visitor.source_transforms
@@ -167,7 +178,9 @@ class SphinxCodeAutoLink:
         }
         inter_inv = InventoryAdapter(app.env).main_inventory
         transposed = transpose_inventory(inter_inv, relative_to=app.outdir)
-        transposed.update(transpose_inventory(inventory, relative_to=app.outdir))
+        transposed.update(
+            transpose_inventory(inventory, relative_to=app.outdir, use_tuple=True)
+        )
         return transposed
 
     @print_exceptions()
@@ -255,7 +268,13 @@ class SphinxCodeAutoLink:
             rm_vis = RemoveExtensionVisitor(doctree)
             return doctree.walkabout(rm_vis)
 
-        visitor = CodeRefsVisitor(doctree, code_refs=self.code_refs)
+        visitor = CodeRefsVisitor(
+            doctree,
+            code_refs=self.code_refs,
+            docname=docname,
+            builder=app.builder,
+            warn_no_backreference=self.warn_no_backreference,
+        )
         doctree.walk(visitor)
         return None
 
@@ -275,12 +294,15 @@ class SphinxCodeAutoLink:
                 self.inventory,
                 self.custom_blocks,
                 self.search_css_classes,
+                builder_name=app.builder.name,
             )
 
         self.cache.write()
 
 
-def transpose_inventory(inv: dict, relative_to: str) -> dict[str, str]:
+def transpose_inventory(
+    inv: dict, relative_to: str, *, use_tuple: bool = False
+) -> dict[str, str]:
     """
     Transpose Sphinx inventory from {type: {name: (..., location)}} to {name: location}.
 
@@ -292,13 +314,18 @@ def transpose_inventory(inv: dict, relative_to: str) -> dict[str, str]:
         Sphinx inventory
     relative_to
         if a local file is found, transform it to be relative to this dir
+    use_tuple
+        force using Sphinx inventory tuple interface,
+        TODO: move to class interface if it becomes public (#173)
     """
     transposed = {}
     for type_, items in inv.items():
         if not type_.startswith("py:"):
             continue
         for item, info in items.items():
-            location = info[2]
+            location = (
+                info.uri if not use_tuple and sphinx_version >= (8, 2) else info[2]
+            )
             if not location.startswith("http"):
                 location = str(Path(location).relative_to(relative_to))
             transposed[item] = location
