@@ -17,7 +17,18 @@ from sphinx_codeautolink.warn import logger, warn_type
 from .backref import CodeExample
 from .directive import ConcatMarker, PrefaceMarker, SkipMarker
 
-BUILTIN_BLOCKS = {"python": None, "py": None}
+# list from https://pygments.org/docs/lexers/#pygments.lexers.python.PythonLexer
+BUILTIN_BLOCKS = {
+    "default": None,
+    "python": None,
+    "python3": None,
+    "py": None,
+    "py3": None,
+    "pyi": None,
+    "sage": None,
+    "bazel": None,
+    "starlark": None,
+}
 
 
 @dataclass
@@ -39,7 +50,7 @@ def clean_pycon(source: str) -> tuple[str, str]:
         if line.startswith(">>> "):
             in_statement = True
             clean_lines.append(line[4:])
-        elif in_statement and line.startswith("... "):
+        elif in_statement and line.startswith("..."):
             clean_lines.append(line[4:])
         else:
             in_statement = False
@@ -94,7 +105,7 @@ else:
 
 
 class CodeBlockAnalyser(nodes.SparseNodeVisitor):
-    """Transform literal blocks of Python with links to reference documentation."""
+    """Transform blocks of Python with links to reference documentation."""
 
     def __init__(
         self,
@@ -103,6 +114,8 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
         global_preface: list[str],
         custom_blocks: dict[str, Callable[[str], str]],
         concat_default: bool,
+        default_highlight_lang: str | None,
+        warn_default_parse_fail: bool,
         **kwargs,
     ) -> None:
         super().__init__(*args, **kwargs)
@@ -110,16 +123,20 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
         relative_path = Path(self.document["source"]).relative_to(source_dir)
         self.current_document = str(relative_path.with_suffix(""))
         self.global_preface = global_preface
+        self.file_preface = []
+        self.prefaces = []
+        self.doctest_preface: dict[str, list[str]] = {}
         self.transformers = BUILTIN_BLOCKS.copy()
         self.transformers.update(custom_blocks)
         self.valid_blocks = self.transformers.keys()
         self.title_stack = []
         self.current_refid = None
-        self.prefaces = []
         self.concat_global = concat_default
         self.concat_section = False
         self.concat_sources = []
         self.skip = None
+        self.highlight_lang = default_highlight_lang
+        self.warn_default_parse_fail = warn_default_parse_fail
 
     def unknown_visit(self, node) -> None:
         """Handle and delete custom directives, ignore others."""
@@ -138,7 +155,16 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
                 self.concat_global = node.mode == "on"
             node.parent.remove(node)
         elif isinstance(node, PrefaceMarker):
-            self.prefaces.extend(node.content.split("\n"))
+            lines = node.content.split("\n") or []
+            if node.level == "next":
+                self.prefaces.extend(lines)
+            elif node.level == "file":
+                self.file_preface = lines
+            else:
+                msg = f"Invalid preface argument: `{node.level}`"
+                logger.error(
+                    msg, type=warn_type, subtype="invalid_argument", location=node
+                )
             node.parent.remove(node)
         elif isinstance(node, SkipMarker):
             if node.level not in ("next", "section", "file", "off"):
@@ -152,6 +178,10 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
     def unknown_departure(self, node) -> None:
         """Ignore unknown nodes."""
 
+    def visit_highlightlang(self, node) -> None:
+        """Set expected highlight language."""
+        self.highlight_lang = node["lang"]
+
     def visit_title(self, node) -> None:
         """Track section names and break concatenation and skipping."""
         self.title_stack.append(node.astext())
@@ -169,34 +199,67 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
         """Pop latest title."""
         self.title_stack.pop()
 
-    def visit_doctest_block(self, node):
+    def visit_doctest_block(self, node: nodes.doctest_block):
         """Visit a Python doctest block."""
         return self.parse_source(node, "pycon")
 
+    def visit_comment(self, node: nodes.comment):
+        """Visit an ext.doctest setup block."""
+        if node.get("testnodetype") == "testsetup":
+            groups = node["groups"]
+            lines = [ln for c in node.children for ln in c.astext().split("\n")]
+            for g in groups:
+                if g == "*":
+                    self.doctest_preface = {"*": lines}
+                else:
+                    self.doctest_preface[g] = lines
+
     def visit_literal_block(self, node: nodes.literal_block):
         """Visit a generic literal block."""
-        return self.parse_source(node, node.get("language", None))
+        return self.parse_source(node, node.get("language", self.highlight_lang))
 
-    def parse_source(
+    def parse_source(  # noqa: C901,PLR0912
         self, node: nodes.literal_block | nodes.doctest_block, language: str | None
     ) -> None:
         """Analyse Python code blocks."""
         prefaces = self.prefaces
         self.prefaces = []
 
+        if node.get("testnodetype") == "doctest":
+            groups = node["groups"]
+            doctest_prefaces = [
+                ln
+                for g in groups
+                for ln in self.doctest_preface.get(g, self.doctest_preface.get("*", []))
+            ]
+            prefaces = doctest_prefaces + prefaces
+
         skip = self.skip
         if skip == "next":
             self.skip = None
 
         if (
-            skip
-            or len(node.children) != 1
+            len(node.children) != 1
             or not isinstance(node.children[0], nodes.Text)
             or language not in self.valid_blocks
         ):
             return
 
         source = node.children[0].astext()
+        example = CodeExample(
+            self.current_document, self.current_refid, list(self.title_stack)
+        )
+        transform = SourceTransform(source, [], example, node.line)
+        self.source_transforms.append(transform)
+
+        if skip:
+            return
+
+        # Sphinx uses a similar trick to use pycon implicitly (#168)
+        pycon_candidates = ("py", "python", "py3", "python3", "default", "pycon3")
+        if source.startswith(">>>") and language in pycon_candidates:
+            language = "pycon"
+
         transformer = self.transformers[language]
         if transformer:
             try:
@@ -209,28 +272,47 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
                 return
         else:
             clean_source = source
-        example = CodeExample(
-            self.current_document, self.current_refid, list(self.title_stack)
-        )
-        transform = SourceTransform(source, [], example, node.line)
-        self.source_transforms.append(transform)
+        transform.source = source
 
-        modified_source = "\n".join(
-            self.global_preface + self.concat_sources + prefaces + [clean_source]
+        full_source = "\n".join(
+            self.global_preface
+            + self.file_preface
+            + self.concat_sources
+            + prefaces
+            + [clean_source]
         )
         try:
-            names = parse_names(modified_source, node)
+            names = parse_names(full_source, node)
         except SyntaxError as e:
+            if language == "default" and not self.warn_default_parse_fail:
+                return
+
             show_source = self._format_source_for_error(
-                self.global_preface, self.concat_sources, prefaces, source
+                self.global_preface,
+                self.file_preface,
+                self.concat_sources,
+                prefaces,
+                transform.source,
             )
             msg = self._parsing_error_msg(e, language, show_source)
             logger.warning(msg, type=warn_type, subtype="parse_block", location=node)
             return
+        except Exception as e:
+            show_source = self._format_source_for_error(
+                self.global_preface,
+                self.file_preface,
+                self.concat_sources,
+                prefaces,
+                transform.source,
+            )
+            msg = self._parsing_error_msg(e, language, show_source)
+            raise type(e)(msg) from e
 
-        if prefaces or self.concat_sources or self.global_preface:
+        if prefaces or self.concat_sources or self.global_preface or self.file_preface:
             concat_lens = [s.count("\n") + 1 for s in self.concat_sources]
-            hidden_len = len(prefaces) + sum(concat_lens) + len(self.global_preface)
+            hidden_len = len(prefaces + self.global_preface + self.file_preface) + sum(
+                concat_lens
+            )
             for name in names:
                 name.lineno -= hidden_len
                 name.end_lineno -= hidden_len
@@ -244,16 +326,26 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
     @staticmethod
     def _format_source_for_error(
         global_preface: list[str],
+        file_preface: list[str],
         concat_sources: list[str],
         prefaces: list[str],
         source: str,
     ) -> str:
-        lines = global_preface + concat_sources + prefaces + source.split("\n")
+        lines = (
+            global_preface
+            + file_preface
+            + concat_sources
+            + prefaces
+            + source.split("\n")
+        )
         guides = [""] * len(lines)
         ix = 0
         if global_preface:
-            guides[0] = "global preface:"
+            guides[ix] = "global preface:"
             ix += len(global_preface)
+        if file_preface:
+            guides[ix] = "file preface:"
+            ix += len(concat_sources)
         if concat_sources:
             guides[ix] = "concatenations:"
             ix += len(concat_sources)
@@ -271,6 +363,7 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
                 str(error) + f" in document {self.current_document!r}",
                 f"Parsed source in `{language}` block:",
                 source,
+                "",
             ]
         )
 
@@ -282,9 +375,14 @@ def link_html(
     inventory: dict,
     custom_blocks: dict,
     search_css_classes: list,
+    builder_name: str = "html",
 ) -> None:
     """Inject links to code blocks on disk."""
-    html_file = Path(out_dir) / (document + ".html")
+    if builder_name == "dirhtml" and Path(document).name != "index":
+        html_file = Path(out_dir) / document / "index.html"
+    else:
+        html_file = Path(out_dir) / (document + ".html")
+
     text = html_file.read_text("utf-8")
     soup = BeautifulSoup(text, "html.parser")
 
